import { Component, OnInit, ViewChild, NgZone } from '@angular/core';
import { Platform, NavController, ModalController, IonMenu } from '@ionic/angular';
import { Plugins} from '@capacitor/core';
const { SplashScreen } = Plugins;
import { StatusBar } from '@ionic-native/status-bar/ngx';
import { HttpService } from './services/http.service';
import { AuthService } from './services/auth.service';
import { UserService } from './services/user.service';
import { MenuController } from '@ionic/angular';
import { PatientService } from './services/patient.service';
import { Observable } from 'rxjs';
import { Patient } from './models/Server';
import { map } from 'rxjs/operators';
import { StateService } from './services/state.service';
import { ExplainerComponent } from './components/notifications/explainer/explainer.component';
import { VideoComponent } from './components/video/video.component';
import { NotificationService } from 'src/app/services/notification.service';
import { SocketsService } from './services/sockets.service';
import { TwilioService } from './services/twilio.service';
import { CommunicationService } from 'src/app/services/communication.service';
import { Router } from '@angular/router';
import { WelcomeScreenComponent } from './components/notifications/welcome-screen/welcome-screen.component';
import {AudioService} from './services/audio.service';

interface IMenuItem {
  title: string;
  icon?: string;
  url?: string;
  roles: string[];
  subMenu?: any[];
}

@Component({
  selector: 'app-root',
  templateUrl: 'app.component.html'
})

export class AppComponent implements OnInit {
  public children$: Observable<Patient[]>;
  public children;
  public currentChild;
  public menuItems$: Observable<IMenuItem[]>;
  public showChildToggle = false;
  organizations$;
  notificationCount = 0;
  showCount = false;
  @ViewChild(IonMenu) sideMenu: IonMenu;

  constructor(
    private platform: Platform,
    private statusBar: StatusBar,
    private httpService: HttpService,
    public authService: AuthService,
    public userService: UserService,
    private menu: MenuController,
    private nav: NavController,
    private patientService: PatientService,
    private stateService: StateService,
    private modalCtrl: ModalController,
    public notificationService: NotificationService,
    private socketsService: SocketsService,
    private twilioService: TwilioService,
    private audio: AudioService,
    public videoComponent: VideoComponent,
    private communicationService: CommunicationService,
    private router: Router,
    private ngZone: NgZone,
  ) {
    this.initializeApp();
  }

  ngOnInit() {
    console.log('app component init');

    setTimeout(() => {
      if (this.userService) {
        if (this.userService.user.last_login === null) {
          this.openWelcomeScreen({type: this.userService.isDoctor});
        }
      }
    }, 2000);

    this.menuItems$ = this.patientService.currentPatient.pipe(
      map((child: Patient) => {
        if (child && child.id) {
          this.currentChild = child;
          const menuItems: IMenuItem[] = [
            {
              title: 'Timeline',
              url: `/timeline`,
              roles: ['guardian'],
              subMenu: undefined
            },
            {
              title: 'Notifications',
              url: '/notifications',
              roles: ['doctor', 'guardian'],
              subMenu: undefined
            },
            {
              title: 'Patients',
              url: '/patients',
              roles: ['doctor'],
              subMenu: undefined
            },
            {
              title: 'Communication Center',
              icon: 'assets/icon/messages.svg',
              url: `/communication`,
              roles: ['doctor'],
              subMenu: undefined
            },
             {
               title: 'Communication Center',
               url: `/communication`,
               roles: ['guardian'],
               subMenu: undefined
             },
            {
              title: 'Tools',
              roles: ['doctor'],
              subMenu: [
                {
                  title: 'Leaderboard',
                  url: '/leaderboard',
                },
                {
                  title: 'Team Members',
                  url: '/settings/organization',
                },
                // {
                //   title: 'Dictation',
                //   url: '/patients',
                // },
                {
                  title: 'Training Overview',
                  url: '/training-overview',
                },
                {
                  title: 'Parent Explainer',
                  url: '/parent-explainer',
                },
              ]
            },
            {
              title: 'Goals',
              url: `/patient/${child.id}/goals`,
              roles: ['guardian']
            },
            {
              title: 'Daily Focus',
              url: `/workouts/${child.id}`,
              roles: ['guardian']
            },
            {
              title: 'Documents',
              url: `/patient/${child.id}/documents`,
              roles: ['guardian']
            },
            {
              title: 'Organizations',
              roles: ['guardian'],
              subMenu: [
                {
                  title: 'Add New',
                  url: '/settings/add-network',
                },
              ]
            },
            {
              title: 'Feedback',
              roles: ['doctor', 'guardian'],
              subMenu: [
                {
                  title: 'Submit a Tag',
                  url: '/submit-a-tag',
                  roles: ['doctor']
                },
                {
                  title: 'Submit Feedback',
                  url: '/submit-feedback',
                  roles: ['doctor', 'guardian']
                },
                {
                  title: '#TherapyWins',
                  url: '/therapy-wins',
                  roles: ['doctor', 'guardian']
                },
              ]
            },
            {
              title: 'Billing and Audit',
              url: `/audit`,
              roles: ['doctor']
            },
            {
              title: 'Settings',
              url: `/settings`,
              roles: ['guardian', 'doctor']
            }
          ];

       /*   if (!this.platform.is('cordova')) {
            menuItems = menuItems.filter((page) => {
              return page.title.indexOf('Communication Center');
            });
          }*/

          const role = this.userService.isDoctor ? 'doctor' : 'guardian';
          const filtered = menuItems.filter((page) => {
            return page.roles.includes(role);
          });
          filtered.forEach((page) => {
            if (!page.subMenu) {
              return;
            }
            page.subMenu = page.subMenu.filter(item => !item.roles || item.roles.includes(role));
          });
          return filtered;
        } else {
          return null;
        }
      })
    );

    this.organizations$ = this.userService.getNetwork().pipe(
      map((data) => {
        return data.organizations;
      })
    );

  }

  initializeApp() {
    this.platform.ready().then(() => {
      // this.statusBar.styleDefault();
      this.httpService.checkNetwork();

      this.stateService.getDeviceInfo();

      this.patientService.patients.subscribe((res) => {
        this.children = res;
      });

      this.notificationService.notificationCount.subscribe((val) => {
        this.notificationCount = val;
        this.showCount = this.notificationCount > 0;
      });

      SplashScreen.hide();
    });
  }

  private get menuSubtitle() {
    return this.userService.isDoctor ? this.userService.user.organizations[0].name : this.userService.user.email;
  }

  selectChild(childId) {
    const selectedChild = this.children.filter((child) => child.id === childId)[0];
    this.patientService.setCurrentPatient(selectedChild);

    this.showChildToggle = false;
    this.nav.navigateForward(`/timeline`);
  }

  joinIncomingCall() {

    const token = document.getElementById('join-incoming-call-notification').dataset.token;
    const roomName = document.getElementById('join-incoming-call-notification').dataset.roomname;
    this.communicationService.setIncomingCall({
      token,
      roomName,
    });

    document.getElementById('socket-notifications').style.display = 'none';
    document.getElementById('incoming-call-notification').style.display = 'none';
    this.videoComponent.stopRing();
    this.ngZone.run(() => this.router.navigate([`/video`])).then();
  }

  declineIncomingCall() {

    const token = document.getElementById('join-incoming-call-notification').dataset.token;
    const roomName = document.getElementById('join-incoming-call-notification').dataset.roomname;
    this.communicationService.setIncomingCall({
      token,
      roomName,
    });

    document.getElementById('socket-notifications').style.display = 'none';
    document.getElementById('incoming-call-notification').style.display = 'none';
    this.ngZone.run(() => this.router.navigate([`/video`])).then();
  }

  navigateToUrl(url) {
    if (url === '/training-overview') {
      this.openExplainerModal({type: 'TRAINING'});
    } else if (url === '/parent-explainer') {
      this.openExplainerModal({type: 'PARENT'});
    } else if (url === '/submit-a-tag') {
      window.open('https://theraweconnect.typeform.com/to/J4QeFO');
    } else if (url === '/submit-feedback') {
      window.open('https://theraweconnect.typeform.com/to/H0dEX1');
    } else if (url === '/therapy-wins') {
      if (this.userService.isDoctor) {
        window.open('https://theraweconnect.typeform.com/to/GQoA1s');
      } else if (this.userService.isGuardian) {
        window.open('https://theraweconnect.typeform.com/to/wkWM80');
      }
    } else if (!!url) {
      this.nav.navigateForward([url]);
    }
  }



  async openExplainerModal(notification) {
    const modal = await this.modalCtrl.create({
      component: ExplainerComponent,
      componentProps: {
        notification
      }
    });

    await modal.present();
  }

  async openWelcomeScreen(type) {
    const modal = await this.modalCtrl.create({
      component: WelcomeScreenComponent,
      componentProps: {
        type
      }
    });

    await modal.present();
  }

  goToSettings() {
    this.nav.navigateForward(`/settings`);
    this.sideMenu.close();
  }
}
