import { Component, OnInit } from '@angular/core';
import { UserService } from 'src/app/services/user.service';
import { tap, map } from 'rxjs/operators';
import { StateService } from 'src/app/services/state.service';
import { ReportsService } from 'src/app/services/reports.service';
import * as moment from 'moment';
import { Papa } from 'ngx-papaparse';
import { Platform } from '@ionic/angular';
import { File } from '@ionic-native/file/ngx';
import { SocialSharing } from '@ionic-native/social-sharing/ngx';
import * as _ from 'lodash';

@Component({
  selector: 'app-audit',
  templateUrl: './audit.component.html',
  styleUrls: ['./audit.component.scss'],
})
export class AuditComponent implements OnInit {
  public pageTitle = 'Insurance Audit Reporting';
  public platform;
  public organizationId;
  public data;
  public startDate;
  public endDate;
  public start;
  public end;

  constructor(
    public userService: UserService,
    public stateService: StateService,
    public reportsService: ReportsService,
    public papa: Papa,
    private plt: Platform,
    private file: File,
    private socialSharing: SocialSharing
  ) { }

  ngOnInit() {
    this.platform = this.stateService.getPlatform();
    this.organizationId = this.userService.user.organizations[0].id;
    this.getCurrentWeek();
  }

  getCurrentWeek() {
    const ourDate = new Date();
    this.setDateValues(ourDate);
    this.getInsurance(this.organizationId, this.start, this.end);
  }

  getPreviousWeek() {
    const pastDate = moment(moment(this.start).subtract(7, 'days')).toDate();
    this.setDateValues(pastDate);
    this.getInsurance(this.organizationId, this.start, this.end);
  }

  getNextWeek() {
    const nextDate = moment(moment(this.start).add(7, 'days')).toDate();
    this.setDateValues(nextDate);
    this.getInsurance(this.organizationId, this.start, this.end);
  }

  setDateValues(date) {
    const week = [];

    for (let i = 1; i <= 7; i++) {
      const first = date.getDate() - date.getDay() + i;
      const day = new Date(date.setDate(first)).toISOString();
      week.push(day);
    }

    this.start = week[0];
    this.end = week[6];
    this.startDate = moment(this.start).format('MMM DD, YYYY');
    this.endDate = moment(this.end).format('MMM DD, YYYY');
  }

  getInsurance(organizationId, start, end) {
    this.reportsService.getInsurance(organizationId, start, end)
    .subscribe((res) => {

      for (const i of res) {
        i.date = moment(i.date_time).format('MM-DD-YYYY');
        i.time = moment(i.date_time).format('LT');
        i.therapist_type = i.therapist_type === null ? '-' : i.therapist_type;
      }

      this.data = res;
    });
  }

  sort(ev) {
    const column = ev.srcElement.title;
    if (ev.srcElement.name === 'md-arrow-dropup') {
      ev.srcElement.name = 'md-arrow-dropdown';
      this.isAscending(this.data, column);
    } else if (ev.srcElement.name === 'md-arrow-dropdown') {
      ev.srcElement.name = 'md-arrow-dropup';
      this.isDescending(this.data, column);
    }
  }

  isAscending(arr, prop) {
    return arr.sort((a, b) => (a[prop] > b[prop]) ? 1 : -1);
  }

  isDescending(arr, prop) {
    return arr.sort((a, b) => (a[prop] > b[prop]) ? -1 : 1);
  }

  exportCSV() {
    const toCSV = [];
    let picked;

    for (const i of this.data) {
      picked = _.pick(i, ['date', 'time', 'patient_name', 'therapist_name', 'therapist_type', 'activity_event']);
      toCSV.push(picked);
    }

    const csv = this.papa.unparse({
      data: toCSV
    });
    const fileName = 'insurance-audit-report-'
    + moment(this.start).format('MMDDYYYY') + '-'
    + moment(this.end).format('MMDDYYYY') + '.csv';

    if (this.plt.is('cordova')) {
      this.file.writeFile(this.file.dataDirectory, fileName, csv, { replace: true }).then(res => {
        this.socialSharing.share(null, null, res.nativeURL, null);
      });
    } else {
      const blob = new Blob([csv]);
      const a = window.document.createElement('a');
      a.href = window.URL.createObjectURL(blob);
      a.download = fileName;
      document.body.appendChild(a);
      a.click();
      document.body.removeChild(a);
    }

  }
}
