import { Component, OnInit, NgZone } from '@angular/core';
import { ActivatedRoute, ParamMap } from '@angular/router';
import { PatientService } from 'src/app/services/patient.service';
import { takeUntil } from 'rxjs/operators';
import { forkJoin } from 'rxjs';
import { Unsubscriber } from '../common/unsubscriber';
import { Patient } from 'src/app/models/Server';
import { GetPatientResponse } from 'src/app/models/Http';
import { ActionSheetController, PopoverController } from '@ionic/angular';
import { Observable } from 'rxjs';
import { IDocument } from 'src/app/models/server/Document';
import { IDocumentsResponse, DocumentService, DocumentUpload } from 'src/app/services/document.service';
import { Plugins, CameraResultType, CameraSource } from '@capacitor/core';
const { Camera, Device } = Plugins;
import { DomSanitizer } from '@angular/platform-browser';
import { DocumentAttachment } from '../../models/client/DocumentAttachment';
import { PopoverMenuComponent } from '../common/popover-menu/popover-menu.component';
import { OverlayEventDetail } from '@ionic/core';
import { UserService } from 'src/app/services/user.service';
import { StateService } from 'src/app/services/state.service';
import { FileService, FileUploadState } from 'src/app/services/file.service';
import { UIFileState } from 'src/app/models/interfaces/ui-file-state';
import { UiUserDocumentModel } from 'src/app/models/client/ui-user-document-model';
import * as _ from 'lodash';
import { ToastService } from 'src/app/services/toast.service';

@Component({
  selector: 'app-documents',
  templateUrl: './documents.component.html',
  styleUrls: ['./documents.component.scss'],
})
export class DocumentsComponent extends Unsubscriber implements OnInit {
  pageTitle = 'Documents';
  patient: Patient;
  documents = [];
  editMode = false;

  constructor(
    private route: ActivatedRoute,
    private patientService: PatientService,
    private documentService: DocumentService,
    private actionCtrl: ActionSheetController,
    private sanitizer: DomSanitizer,
    public popoverController: PopoverController,
    public userService: UserService,
    public fileService: FileService,
    public ngZone: NgZone,
    public toast: ToastService,
    public stateService: StateService
  ) {
    super();
  }

  ngOnInit() {
    this.patient = this.patientService.currentPatient.value;

    this.fileService.downloadFinish$.pipe(takeUntil(this._unsubscriber))
      .subscribe((arg) => {
        if (!arg.success) {
          console.error('DL FAILED');
          console.error(arg);
          return;
        }

        this.ngZone.run(() => {
          const doc = this.getDocumentById(arg.key);
          if (doc) {
            doc.state = UIFileState.Exists;
            doc.progress = 0;
            this.toast.presentToast(`Downloaded ${doc.name}`);
            if (arg.meta.autoOpen) {
              this.openFile(doc);
            }
          }
        });

      });


    this.patientService.currentPatient
      .pipe(takeUntil(this._unsubscriber))
      .subscribe(() => {
        this.documentService.getDocuments();
      });

    this.documentService.uploadChange$
      .pipe(takeUntil(this._unsubscriber))
      .subscribe(upload => {
        switch (upload.state) {
          case FileUploadState.Start:
            return this.handleUploadStart(upload);
          case FileUploadState.Progress:
            return this.handleUploadProgress(upload);
          case FileUploadState.Success:
            return this.handleUploadSuccess(upload);
          case FileUploadState.Error:
            return this.handleUploadError(upload);
          default:
            break;
        }
      });

    this.documentService.documents.subscribe((docs) => {
      this.documents = docs;
    });

    this.documentService.getDocuments();
  }

  async openAttachmentActionSheet() {
    const actionSheet = await this.actionCtrl.create({
      mode: 'ios',
      header: 'Upload Document',
      subHeader: 'Please select one of the options below',
      cssClass: 'attachmentSheet',
      buttons: [
        {
          text: 'Files',
          icon: 'document',
          handler: () => {
            // Attach file here
            // this.addDocument(this.mockDocuments);
            this.selectDocument();
          }
        },
        {
          text: 'Cancel',
          role: 'cancel',
          handler: () => {
            console.log('Cancel clicked');
          }
        }
      ]
    });
    await actionSheet.present();
  }


  private getDocumentById(documentId: string) {
    return _.find(this.documents, (d) => d.id === documentId);
  }

  onMultiSelect(items: any, i: any) {
    const document = this.documents[i];
    if (this.editMode) {
      items[i].isChecked = true;
      this.editMode = true;
      return;
    } else if (!this.editMode) {
      if (this.stateService.getPlatform() === 'web') {
        this.downloadFile(document, false, true);
      } else {
        console.log('DOCUMENT TO OPEN: ' + JSON.stringify(document));
        if (document.state === UIFileState.Exists) {
          return this.openFile(document);
        }
        this.downloadFile(document);
      }
    }
  }

  private downloadFile(document: UiUserDocumentModel, autoOpen = true, web = false) {
    console.log('DOWNLOADING FILE..');
    this.ngZone.run(() => {
      document.state = UIFileState.Downloading;
    });

    this.fileService.getDocumentDownloadUrl(document.id).pipe(takeUntil(this._unsubscriber))
      .subscribe((result) => {
        if (this.stateService.getPlatform() === 'web') {
          window.open(result.url, '_blank');
        } else {
          this.fileService.downloadUrl(result.url, document.file.filename, document.id, { autoOpen });
        }
      });
  }

  public async openFile(document: any) {
    console.log('Opening file...');
    try {
      await this.fileService.openDownloadedFile(document.file.filename, document.file.mime_type);
    } catch (e) {
      console.error(e);
      this.toast.presentToast('Failed to open file.');
    }
  }

  onChange(items: any) {
    const noSelect = Object.values(items).every((item: any) => !item.isChecked);
    if (noSelect) {
      this.editMode = false;
    }
  }


  selectAllDocuments(items: any, showEditMode: boolean, isChecked: boolean) {
    this.editMode = showEditMode;
    items.forEach((item: any) => {
      item.isChecked = isChecked;
    });
  }

  async showMenu(ev: any, items: any) {
    const menu = (items.some((i: any) => i.isChecked)) ? [
      { text: 'Clear Selection', icon: 'remove-circle' }
    ] : [
      { text: 'Select...', icon: 'checkmark-circle-outline' },
      { text: 'Select all', icon: 'checkmark-circle' }
    ];
    const popover = await this.popoverController.create({
      component: PopoverMenuComponent,
      componentProps: {
        menu,
        items
      },
      event: ev,
      translucent: true
    });
    popover.onDidDismiss().then((detail: OverlayEventDetail) => {
      if (detail !== null) {
        if (detail.data === 'Select...') {
          this.editMode = true;
        }
        if (detail.data === 'Clear Selection') {
          this.selectAllDocuments(items, false, false);
        }
        if (detail.data === 'Select all') {
          this.selectAllDocuments(items, true, true) ;
        }
      }
    });

    return await popover.present();
  }

  async deleteDocument(ev: any, items: any) {
    const filesToDelete = items.filter((item: any) => item.isChecked);
    const deleteFns = filesToDelete.map((item: any) => this.documentService.deleteDocument(item.id));
    forkJoin(deleteFns).subscribe(results => {
      this.toast.presentToast('Files deleted successfully');
      const filtered = items.filter((item: any) => !item.isChecked);
      this.editMode = false;
      this.documents = filtered;
    },
    error => {
      console.error(error);
      this.toast.presentToast('Error deleting files!');
    });
  }

  async addDocument(items: any) {
    this.selectDocument();
  }

  private async selectDocument() {
    try {
      console.log('picking file...');
      const pickedFile = await this.fileService.filePicker.pickFile();
      this.uploadSelectedFile(pickedFile);
    } catch (e) {
      console.error(e);
    }
  }

  public handleFiles(event) {
    const file = event.target.files[0];
    this.uploadSelectedFile(file);
  }


  private handleUploadStart(upload: DocumentUpload) {
    const curUploadIdx = _.findIndex(this.documents, d => d.id === upload.meta.tmpKey);
    const curUpload = this.documents[curUploadIdx];

    this.documents[curUploadIdx] = Object.assign({}, curUpload, {
      id: upload.key,
      file: upload.file
    });
  }

  // find and update the document being uploaded
  private handleUploadProgress(upload: DocumentUpload) {
    const curUploadIdx = _.findIndex(this.documents, d => d.id === upload.key);
    this.documents[curUploadIdx].progress = upload.progress;
  }

  private handleUploadSuccess(upload: DocumentUpload) {
    const curUploadIdx = _.findIndex(this.documents, d => d.id === upload.key);
    const curUpload = this.documents[curUploadIdx];

    console.log('handling success: ' + JSON.stringify(curUpload));

    this.documents[curUploadIdx] = Object.assign({}, curUpload, {
      state: UIFileState.Ready,
      progress: 0,
      id: upload.documentId
    });
  }

  private handleUploadError(upload: DocumentUpload) {
    console.error('handleUploadError');
    console.error(upload.error);
    this.toast.presentToast('Failed during file upload');
    this.removeDocumentFromList(upload.meta.tmpKey);
  }

  private removeDocumentFromList(documentId: string) {
    const docsCpy = this.documents.slice(0);
    _.remove(docsCpy, (d) => d.id === documentId);
    this.documents = docsCpy;
  }

  private uploadSelectedFile(pickedFile: string) {
    const fileUpload = this.documentService.uploadDocument(pickedFile, this.patientService.currentPatient.value.id);

    this.documents.unshift({
      id: fileUpload.meta.tmpKey,
      name: fileUpload.fileName,
      state: UIFileState.Pending
    });
  }
}

