import { Injectable } from '@angular/core';
import { StateService } from './state.service';
import { IOSFilePicker } from '@ionic-native/file-picker/ngx';
import { AndroidFilePicker, FilePicker, IosFilePicker } from './file-picker';
import { FileTransfer, FileTransferObject, FileUploadOptions, FileUploadResult } from '@ionic-native/file-transfer/ngx';
import { File, DirectoryEntry, FileEntry, IFile } from '@ionic-native/file/ngx';
import { FileOpener } from '@ionic-native/file-opener/ngx';

import { FileChooser } from '@ionic-native/file-chooser/ngx';
import { FilePath } from '@ionic-native/file-path/ngx';
import { FileModel } from './session.service';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { environment } from 'src/environments/environment';
import { Observable, Subject } from 'rxjs';
import { SimpleQueue } from '../services/utils/simple-queue';
import * as _ from 'lodash';

export interface CodedError extends Error {
  code?: number;
}
export interface FileUpload {
  filePath?: string;
  fileName: string;
  file?: FileModel;
  key: string;
  progress: number;
  state: FileUploadState;
  mimeType?: string;
  meta?: any;
  error?: CodedError;
  duration?: number;
  fileSize?: number;

  options?: FileUploadOptions;
  success?: boolean;
  fn?: string;
  fp?: string;
  url?: string;
  ft?: FileTransferObject;
}

export enum FileUploadState {
  Ready,
  Start,
  Progress,
  Success,
  Error
}

interface FileDownload {
  url: string;
  fileName: string;
  downloadPath: string;
  key: string;
  meta?: any;
}
export interface FileUploadUrlResult {
  url: string;
  mime: string;
  file: FileModel;
}

let TRANSFER_ID = 0;


@Injectable({
  providedIn: 'root'
})
export class FileService {
  public filePicker: FilePicker;

  storageDirectory = '';
  sessionsDirectory = 'sessions/';
  public uploadStarted$: Observable<FileUpload>;
  public uploadFinish$: Observable<FileUpload>;
  public uploadProgress$: Observable<FileUpload>;
  public uploadError$: Observable<FileUpload>;
  public downloadStarted$: Observable<any>;
  public downloadFinish$: Observable<any>;
  public downloadProgress$: Observable<any>;
  private _downloadingFiles: any = {};
  private _activeFileTransfer: FileTransferObject;
  private _activeFileTransfers: Array<FileTransferObject>;
  private downloadQueue: SimpleQueue<FileDownload>;
  private uploadQueue: SimpleQueue<FileUpload>;
  private uploadStartedSrc: Subject<FileUpload>;
  private uploadFinishSrc: Subject<FileUpload>;
  private uploadProgressSrc: Subject<FileUpload>;
  private uploadErrorSrc: Subject<FileUpload>;
  private downloadFinishedSrc: Subject<any>;
  private downloadStartedSrc: Subject<any>;
  private downloadProgressSrc: Subject<any>;

  constructor(
    // Disable tslint for FileTransfer
    // tslint:disable-next-line
    private fileTransfer: FileTransfer,
    public stateService: StateService,
    public iosFilePicker: IOSFilePicker,
    public fileChooser: FileChooser,
    public filePath: FilePath,
    public http: HttpClient,
    public file: File,
    public fileOpener: FileOpener

  ) {

    this._activeFileTransfers = [];

    this.uploadStartedSrc = new Subject<FileUpload>();
    this.uploadFinishSrc = new Subject<FileUpload>();
    this.uploadProgressSrc = new Subject<FileUpload>();
    this.uploadErrorSrc = new Subject<FileUpload>();

    this.downloadStartedSrc = new Subject<any>();
    this.downloadFinishedSrc = new Subject<any>();
    this.downloadProgressSrc = new Subject<any>();

    this.uploadStarted$ = this.uploadStartedSrc.asObservable();
    this.uploadFinish$ = this.uploadFinishSrc.asObservable();
    this.uploadProgress$ = this.uploadProgressSrc.asObservable();
    this.uploadError$ = this.uploadErrorSrc.asObservable();

    this.downloadStarted$ = this.downloadStartedSrc.asObservable();
    this.downloadFinish$ = this.downloadFinishedSrc.asObservable();
    this.downloadProgress$ = this.downloadProgressSrc.asObservable();

    this.downloadQueue = new SimpleQueue((download) => this.processDownloadQueue(download));
    this.uploadQueue = new SimpleQueue((upload) => this.processUploadQueue(upload));

    if (this.stateService.getPlatform() === 'ios') {
      this.filePicker = new IosFilePicker(this.iosFilePicker);
    } else if (this.stateService.getPlatform() === 'android') {
      this.filePicker = new AndroidFilePicker(this.fileChooser, this.filePath);
    }
  }
  get nextTransferId() {
    return TRANSFER_ID += 1;
  }

  public uploadFileAsync(file: any, meta?: any, duration: number = 0, fileSize: number = 0) {
    let upload: FileUpload;
    if (typeof file === 'string') {
      upload = {
        filePath: file,
        fileName: file.substring(file.lastIndexOf('/') + 1),
        key: '',
        progress: 0,
        meta,
        state: FileUploadState.Ready,
        duration,
        fileSize
      };
    } else {
      upload = {
        file,
        fileName: file.name,
        key: '',
        progress: 0,
        meta,
        state: FileUploadState.Ready,
        duration,
        fileSize
      };
    }

    console.log(JSON.stringify(upload));
    this.processUploadQueue(upload);
    return upload;
  }

  private processUploadQueue(upload: FileUpload) {
    return new Promise((resolve, reject) => {

      const fileName = upload.filePath ? upload.filePath.substring(upload.filePath.lastIndexOf('/') + 1) : upload.fileName;

      const sub = this.requestFileUploadUrl(fileName, upload.duration, upload.fileSize)
        .subscribe((result) => {

          if (!upload.file && upload.filePath) {
            // mobile upload process
            upload.key = result.file.id;
            upload.state = FileUploadState.Start;
            upload.mimeType = result.mime;
            upload.file = result.file;

            const transferId = this.nextTransferId;
            this._activeFileTransfer = this.fileTransfer.create();
            (this._activeFileTransfer as any).$id = transferId;
            this._activeFileTransfers.push(this._activeFileTransfer);

            const options: FileUploadOptions = {
              fileKey: 'file',
              fileName,
              httpMethod: 'PUT',
              mimeType: result.mime,
              chunkedMode: false,
              headers: {
                'Content-Type': result.mime
              }
            };

            this.uploadStartedSrc.next(upload);

            this._activeFileTransfer.onProgress((evt) => this.onFileTransferProgress(evt, upload));

            this._activeFileTransfer.upload(upload.filePath, result.url, options)
              .then((evt) => {
                console.log('UPLOAD COMPLETE');
                this.removeTransferById(transferId);
                this.onFileTransferComplete(evt, upload);
                sub.unsubscribe();
                resolve();
              })
              .catch((err) => {
                console.error('UPLOAD ERROR', err);
                this.removeTransferById(transferId);
                this.onFileTransferError(err, upload);
                sub.unsubscribe();
                resolve();
              });
          } else if (upload.file) {
            upload.key = result.file.id;
            upload.state = FileUploadState.Start;
            upload.mimeType = result.mime;

            // todo: kjm complete web upload process
            this.http.put(result.url, upload.file, { headers: new HttpHeaders({'x-amz-meta-fileid' : result.file.id})})
            .subscribe((res) => {
              console.log('WEB UPLOAD COMPLETE');
              // this.removeTransferById(transferId);
              this.onFileTransferComplete(null, upload);
              sub.unsubscribe();
              resolve();
            }, (err) => {
              console.log(err);
            });
          }
        }, err => {
          this.onFileTransferError(err, upload);
          sub.unsubscribe();
          resolve();
        });
    });
  }

  public requestFileUploadUrl(fileName: string, duration: number = 0, fileSize: number = 0) {
    return this.http.post<FileUploadUrlResult>(`${environment.API_URL}/api/files`, { fileName, duration, fileSize });
  }

  get downloadDir() {
    return `${this.file.dataDirectory}downloads`;
  }

  public openDownloadedFile(fileName: string, mimeType: string) {
    console.log('OPENING FILE...');
    return this.fileOpener.open(`${this.downloadDir}/${fileName}`, mimeType);
  }

  public getDocumentDownloadUrl(documentId: string): Observable<any> {
    return this.http.get(`${environment.API_URL}/api/documents/${documentId}/download`);
  }

  public downloadUrl(url: string, fileName: string, key?: string, meta?: any) {
    this._downloadingFiles[key] = 0;

    this.downloadQueue.enqueue({
      url,
      fileName,
      downloadPath: `${this.downloadDir}/${fileName}`,
      key,
      meta
    });
  }

  private processDownloadQueue(download: FileDownload) {
    return new Promise((resolve, reject) => {
      const url = download.url;
      const key = download.key;
      const downloadPath = download.downloadPath;
      const meta = download.meta;

      const downloadTx = this.fileTransfer.create();

      downloadTx.onProgress((prog) => {
        const percent = Math.round((prog.loaded / prog.total) * 100);
        this._downloadingFiles[key] = percent;
        this.downloadProgressSrc.next(Object.assign({ key, percent }, prog));
      });

      downloadTx.download(url, downloadPath)
        .then((arg) => {
          delete this._downloadingFiles[key];
          this.downloadFinishedSrc.next({
            success: true,
            key,
            meta
          });
          resolve();
        })
        .catch((err) => {
          delete this._downloadingFiles[key];
          this.downloadFinishedSrc.next({
            success: false,
            key,
            error: err,
            meta
          });
          resolve();
        });

      this.downloadStartedSrc.next({
        key,
        meta
      });
    });
  }

  private removeTransferById(id: number) {
    _.remove(this._activeFileTransfers, (transfer: any) => transfer.$id === id);
  }

  private onFileTransferError = (evt: any, upload: FileUpload) => {
    upload.error = evt;
    upload.state = FileUploadState.Error;
    this.uploadErrorSrc.next(upload);

    // cleanup
    this._activeFileTransfer = undefined;
  }

  private onFileTransferComplete = (evt: FileUploadResult, upload: FileUpload) => {
    upload.state = FileUploadState.Success;
    this.uploadFinishSrc.next(upload);

    // cleanup
    this._activeFileTransfer = undefined;
  }

  private onFileTransferProgress = (evt: ProgressEvent, upload: FileUpload) => {
    upload.progress = (evt.loaded / evt.total) * 100;
    upload.state = FileUploadState.Progress;
    this.uploadProgressSrc.next(upload);
  }


}
