import { Injectable, NgZone } from '@angular/core';
import { StateService } from '../state.service';
import { MediaService } from '../media.service';
import { MediaCapture, MediaFile, CaptureError, CaptureVideoOptions } from '@ionic-native/media-capture/ngx';
import { Plugins, CameraResultType, CameraSource } from '@capacitor/core';
import { ToastService } from '../toast.service';
import { FileService } from '../file.service';
import { Unsubscriber } from '../../components/common/unsubscriber';
import { environment } from 'src/environments/environment';
import { HttpClient } from '@angular/common/http';
import { Observable, forkJoin } from 'rxjs';
import { map, takeUntil } from 'rxjs/operators';
import { FileTransferService } from '../file-transfer.service';

const { Camera } = Plugins;
declare var PKVideoThumbnail: any;


export enum FileState {
  complete = 'COMPLETE',
  pending = 'PENDING'
}

export interface FileModel {
  id: string;
  path: string;
  filename: string;
  original_filename?: string;
  created_by_user_id: string;
  state: FileState;
  mime_type?: string;
  created_at: string;
  updated_at: string;
  duration?: number;
  file_size?: number;
}
export interface FileUploadUrlResult {
  url: string;
  mime: string;
  file: FileModel;
  uploadInfo: any;
}

export enum UploadType {
  WebSessionAttachment = 'webSessionAttachment',
  SessionAttachment = 'sessionAttachment',
  CommentAttachment = 'commentAttachment',
  Document = 'document'
}

export enum ContentType {
  ImageCapture = 'imageCapture',
  VideoCapture = 'videoCapture',
  MediaSelection = 'mediaSelection',
  Document = 'document',
}

export enum PluginContentModel {
  MediaFile = 'mediaFile',
  CameraPhoto = 'cameraPhoto'
}

@Injectable({
  providedIn: 'root'
})
export class UploadService extends Unsubscriber {

  constructor(
    private ngZone: NgZone,
    private stateService: StateService,
    private mediaService: MediaService,
    private mediaCapture: MediaCapture,
    private fileService: FileService,
    private fileTransferService: FileTransferService,
    private toast: ToastService,
    private http: HttpClient
    ) {
      super();
    }

  public async selectContent(contentType: ContentType) {
    let file: any;
    if (this.stateService.getPlatform() === 'web') {
      return;
    } else if (this.stateService.getPlatform() === 'ios' || this.stateService.getPlatform() === 'android') {
      console.log('STARTING MOBILE CONTENT SELECTION...');
      switch (contentType) {
        case ContentType.ImageCapture:
          file = this.startPhotoCapture();
          break;
        case ContentType.VideoCapture:
          file = this.startVideoCapture();
          break;
        case ContentType.MediaSelection:
          console.log('OPENING PHOTO LIBRARY..');
          file = await this.mediaService.selectFromPhotoLibrary();
          break;
        case ContentType.Document:
          file = this.startDocumentSelection();
          break;
      }
    }
    return file;
  }

  private async startVideoCapture() {
    const options: CaptureVideoOptions = { limit: 1 };
    try {
      const videos: MediaFile[] | CaptureError = await this.mediaCapture.captureVideo(options);
      return videos[0];
    } catch (err) {
      console.error(err);
      this.toast.presentToast('Upload failed, please try again!');
    }
  }

  private async startPhotoCapture() {
    const image = await Camera.getPhoto({
      quality: 90,
      allowEditing: false,
      resultType: CameraResultType.Uri,
      source: CameraSource.Camera
    });
    return image;
  }

  private async startDocumentSelection() {
    try {
      const selectedFile = await this.fileService.filePicker.pickFile();
      return selectedFile;
    } catch (err) {
      this.toast.presentToast('Unable to select Document, please try again!');
    }
  }

  public startContentUpload(file, uploadType: UploadType, context: any = {}) {
    return new Observable((subscriber) => {
      const requestFileUploadPostUrl = this.requestFileUploadPostUrl(file.path, 0, 0);
      const sessionAttachment = forkJoin([requestFileUploadPostUrl])
        .pipe(
          map((uploadUrlResponse) => {
            return this.formatUploadObject(uploadUrlResponse, uploadType, file, context);
          }))
        .pipe(
          takeUntil(this._unsubscriber)).
        subscribe((uploadObject) => {
          console.log(JSON.stringify(uploadObject));
          const uploadId = this.fileTransferService.uploadFile(uploadObject.video, this.ngZone);
          uploadObject.attachmentObject.uploadId = uploadId;
          subscriber.next(uploadObject);
          subscriber.complete();
        }, (err) => {
          subscriber.error(err);
          this.toast.presentToast(
            'There was a problem uploading media, please try again. If you are uploading a long video, we recommend connecting to WiFi.');
        });
    });
  }

  public requestFileUploadPostUrl(fileName: string, duration: number = 0, fileSize: number = 0) {
    console.log('REQUESTING URL', fileName, duration, fileSize);
    return this.http.post<FileUploadUrlResult>(`${environment.API_URL}/api/files/postfile`, { fileName, duration, fileSize });
  }

  public formatUploadObject(uploadUrlResponse, uploadType: UploadType, attachment, context) {
    let fileS3;
    if (uploadUrlResponse && uploadUrlResponse.length > 0) {
      fileS3 = uploadUrlResponse[0];
    }
    let attachmentObject;

    switch (uploadType) {
      case UploadType.WebSessionAttachment:
        attachmentObject = {
          video_file_id: attachment.key,
          thumbnail_file_id: null,
          session_id: context.sessionId || null,
          session_sub_type: 'Default',
          tags: [],
          uploadId: null
        };
        break;
      case UploadType.CommentAttachment:
        if (this.stateService.getPlatform() === 'web') {
          attachmentObject = {
            video_file_id: attachment.key,
            thumbnail_file_id: null,
            session_id: context.sessionId || null,
            uploadId: null
          };
        } else {
          attachmentObject = {
            video_file_id: fileS3.file.id,
            thumbnail_file_id: null,
            uploadId: null
          };
        }
        break;
      case UploadType.SessionAttachment:
        attachmentObject = {
          video_file_id: fileS3.file.id,
          thumbnail_file_id: null,
          session_id: context.sessionId || null,
          session_sub_type: attachment.category,
          tags: attachment.tags,
          uploadId: null
        };
        break;
      case UploadType.Document:
        // todo: kjm - map attachment object for document upload
        break;
    }
    return {
      video: {
        localFilePath: attachment.path,
        uploadUrl: fileS3 ? fileS3.uploadInfo.url : null,
        fileName: fileS3 ? fileS3.file.filename : null,
        mimeType: fileS3 ? fileS3.file.mime_type : null,
        fields: fileS3 ? fileS3.uploadInfo.fields : null,
      },
      thumb: null,
      attachmentObject
    };
  }
}
